from logging import DEBUG, ERROR, INFO, WARNING

from .helpers import init, install_additional_formats, silence
from .logger import *  # pylint: disable=wildcard-import

WARN = WARNING

d = debug
i = info
w = warn = warning
e = error
c = critical
exc = exception



# pylint: disable=redefined-outer-name,unused-variable,expression-not-assigned,singleton-comparison

import pytest

import log


@pytest.mark.parametrize('name, levelname', [
    ('c', 'CRITICAL'),
    ('critical', 'CRITICAL'),
    ('d', 'DEBUG'),
    ('debug', 'DEBUG'),
    ('e', 'ERROR'),
    ('error', 'ERROR'),
    ('exc', 'ERROR'),
    ('exception', 'ERROR'),
    ('i', 'INFO'),
    ('info', 'INFO'),
    ('w', 'WARNING'),
    ('warn', 'WARNING'),
    ('warning', 'WARNING'),
])
def test_level_mapping(expect, caplog, name, levelname):
    getattr(log, name)("message")
    expect(caplog.records[-1].levelname) == levelname



"""Implements the "magic" to create `logging` records for the caller."""

import inspect
import logging
import sys

from . import helpers, state


def create_logger_record(level, message, *args, exc_info=None, **kwargs):
    if 'pytest' in sys.modules:
        root = logging.getLogger()
        helpers.install_additional_formats(root)
    elif not state.initialized:
        helpers.init()

    frame = inspect.currentframe().f_back.f_back.f_back

    module_name = kwargs.pop('module_name', frame.f_globals['__name__'])
    parent_module_name = module_name.split('.')[0]

    logger = logging.getLogger(module_name)
    if not logger.level:
        parent_logger = logging.getLogger(parent_module_name)
        logger.level = parent_logger.level
    if not logger.level:
        logger.level = state.default_level

    if not logger.isEnabledFor(level):
        return False

    record = logger.makeRecord(
        module_name,
        level,
        fn=frame.f_globals['__file__'],
        lno=frame.f_lineno,
        msg=message,
        args=args,
        exc_info=exc_info,
        extra=kwargs,
        sinfo=None,
    )
    logger.handle(record)
    return True



# pylint: disable=redefined-outer-name,unused-variable,expression-not-assigned,singleton-comparison

import logging

from log import state, utils


def describe_create_logger_record():

    def it_uses_the_default_log_level_for_new_loggers(expect):
        state.default_level = logging.INFO

        expect(utils.create_logger_record(
            logging.DEBUG, 'hello', module_name='new_module'
        )) == False

        expect(utils.create_logger_record(
            logging.INFO, 'hello', module_name='new_module'
        )) == True

        expect(utils.create_logger_record(
            logging.WARNING, 'hello', module_name='new_module'
        )) == True

    def it_inherits_the_parent_logging_level(expect):
        state.default_level = logging.INFO

        logger = logging.getLogger('parent_module')
        logger.level = logging.WARNING

        expect(utils.create_logger_record(
            logging.DEBUG, 'hello', module_name='parent_module.new_module'
        )) == False

        expect(utils.create_logger_record(
            logging.INFO, 'hello', module_name='parent_module.new_module'
        )) == False

        expect(utils.create_logger_record(
            logging.WARNING, 'hello', module_name='parent_module.new_module'
        )) == True



# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['log', 'log.tests']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'minilog',
    'version': '1.2.6',
    'description': 'Minimalistic wrapper for Python logging.',
    'long_description': '[![Unix Build Status](https://img.shields.io/travis/jacebrowning/minilog/develop.svg?label=unix)](https://travis-ci.org/jacebrowning/minilog)\n[![Windows Build Status](https://img.shields.io/appveyor/ci/jacebrowning/minilog/develop.svg?label=windows)](https://ci.appveyor.com/project/jacebrowning/minilog)\n<br>\n[![Coverage Status](https://img.shields.io/coveralls/jacebrowning/minilog/develop.svg)](https://coveralls.io/r/jacebrowning/minilog)\n[![Scrutinizer Code Quality](https://img.shields.io/scrutinizer/g/jacebrowning/minilog.svg)](https://scrutinizer-ci.com/g/jacebrowning/minilog/?branch=develop)\n<br>\n[![PyPI Version](https://img.shields.io/pypi/v/minilog.svg)](https://pypi.org/project/minilog)\n[![PyPI License](https://img.shields.io/pypi/l/minilog.svg)](https://pypi.org/project/minilog) \n\n# Overview\n\nEvery project should utilize logging, but for simple use cases, this requires a bit too much boilerplate. Instead of including all of this in your modules:\n\n```python\nimport logging \n\nlogging.basicConfig(\n    level=logging.INFO,\n    format="%(levelname)s: %(name)s: %(message)s",\n)\n\nlog = logging.getLogger(__name__)\n\ndef greet(name):\n    log.info("Hello, %s!", name)\n```\n\nwith this package you can simply:\n\n```python\nimport log\n\ndef greet(name):\n    log.info("Hello, %s!", name)\n```\n\nIt will produce the exact same standard library `logging` records behind the scenes.\n\n# Installation\n\n```sh\n$ pip install minilog\n```\n',
    'author': 'Jace Browning',
    'author_email': 'jacebrowning@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://pypi.org/project/minilog',
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.4,<4.0',
}


setup(**setup_kwargs)
